<?php declare(strict_types=1);

namespace Test\Integration\SQLite;

use BitBadger\PDODocument\{Field, Find, RemoveFields};
use PHPUnit\Framework\Attributes\TestDox;
use PHPUnit\Framework\TestCase;
use Test\Integration\TestDocument;

/**
 * SQLite integration tests for the RemoveFields class
 */
#[TestDox('Remove Fields (SQLite integration)')]
class RemoveFieldsTest extends TestCase
{
    /** @var string Database name for throwaway database */
    private string $dbName;

    protected function setUp(): void
    {
        parent::setUp();
        $this->dbName = ThrowawayDb::create();
    }

    protected function tearDown(): void
    {
        ThrowawayDb::destroy($this->dbName);
        parent::tearDown();
    }

    #[TestDox('By ID succeeds when fields are removed')]
    public function testByIdSucceedsWhenFieldsAreRemoved(): void
    {
        RemoveFields::byId(ThrowawayDb::TABLE, 'two', ['sub', 'value']);
        $doc = Find::byId(ThrowawayDb::TABLE, 'two', TestDocument::class);
        $this->assertNotFalse($doc, 'There should have been a document returned');
        $this->assertEquals('', $doc->value, 'Value should have been blank (its default value)');
        $this->assertNull($doc->sub, 'Sub-document should have been null');
    }

    #[TestDox('By ID succeeds when a field is not removed')]
    public function testByIdSucceedsWhenAFieldIsNotRemoved(): void
    {
        RemoveFields::byId(ThrowawayDb::TABLE, 'one', ['a_field_that_does_not_exist']);
        $this->assertTrue(true, 'The above not throwing an exception is the test');
    }

    #[TestDox('By ID succeeds when no document is matched')]
    public function testByIdSucceedsWhenNoDocumentIsMatched(): void
    {
        RemoveFields::byId(ThrowawayDb::TABLE, 'fifty', ['sub']);
        $this->assertTrue(true, 'The above not throwing an exception is the test');
    }

    public function testByFieldsSucceedsWhenAFieldIsRemoved(): void
    {
        RemoveFields::byFields(ThrowawayDb::TABLE, [Field::EQ('num_value', 17)], ['sub']);
        $doc = Find::firstByFields(ThrowawayDb::TABLE, [Field::EQ('num_value', 17)], TestDocument::class);
        $this->assertNotFalse($doc, 'There should have been a document returned');
        $this->assertNull($doc->sub, 'Sub-document should have been null');
    }

    public function testByFieldsSucceedsWhenAFieldIsNotRemoved(): void
    {
        RemoveFields::byFields(ThrowawayDb::TABLE, [Field::EQ('num_value', 17)], ['nada']);
        $this->assertTrue(true, 'The above not throwing an exception is the test');
    }

    public function testByFieldsSucceedsWhenNoDocumentIsMatched(): void
    {
        RemoveFields::byFields(ThrowawayDb::TABLE, [Field::NE('missing', 'nope')], ['value']);
        $this->assertTrue(true, 'The above not throwing an exception is the test');
    }
}
